#! -*- coding: utf-8 -*-
import datetime
import time
import random
import re
import logging

from django.conf import settings
from django.db import models
from django.contrib.auth.models import User
from django.db.models.signals import post_save, pre_delete
from django.conf import settings
from django.db import transaction
from django.template.loader import render_to_string
from django.utils.hashcompat import sha_constructor
from django.utils.translation import ugettext_lazy as _
from django.core.mail import EmailMultiAlternatives
from mailup.clients import MailUpClientSingleton
from mailup.exceptions import RecipientNotFoundException
from mailup.providers import MailUpComponentProvider

from registration.signals import user_registered
from sccore.profiles.models import SCProfiles, SCProfile

from acomea.soap.models import AcomeaClientCms
from acomea.areariservata.soap import AcomeaClientRiservataCms

__author__ = 'simone cittadini'


log = logging.getLogger("acomea.utenti")


SHA1_RE = re.compile('^[a-f0-9]{40}$')

STATI_RICHIESTA_CLIENTE = (
    ('1', 'nessuna richiesta'),
    ('15', 'codice fiscale verificato'),

    ('2', 'in richiesta cliente consultivo'),
    ('4', 'in richiesta cliente dispositivo'),

    ('3', 'in richiesta cliente consultivo (non accettata)'), # backoffice
    ('5', 'in richiesta cliente dispositivo (non accettata)'), # backoffice

    ('14', 'richiesta consultivo approvato'),
    ('8', 'richiesta dispositivo approvato'),

    ('6', 'posizione consultiva sbloccata'),
    ('9', 'posizione dispositiva sbloccata'),

    ('7', 'contratto dispositivo ricevuto'),
    ('10', 'in richiesta abilitazione dispositivo'),
    ('11', 'in richiesta abilitazione dispositivo (non accettata)'),
    ('12', 'contratto abilitazione dispositivo ricevuto'),
    ('13', 'contratto consultivo accettato'),
)

STATI_RAC = {
    '1' : 'nessuna richiesta',

    '2' : 'in richiesta cliente consultivo',
    '4' : 'in richiesta cliente dispositivo',

    '3' : 'in richiesta cliente consultivo (non accettata)',
    '5' : 'in richiesta cliente dispositivo (non accettata)',

    '14': 'contratto consultivo approvato',
    '8' : 'contratto dispositivo approvato',

    '6' : 'posizione consultiva sbloccata',
    '9' : 'posizione dispositiva sbloccata',

    '7' : 'contratto dispositivo ricevuto',
    '10': 'in richiesta abilitazione dispositivo',
    '11': 'in richiesta abilitazione dispositivo (non accettata)',
    '12': 'contratto abilitazione dispositivo ricevuto',
    '13': 'contratto consultivo accettato',
}

VALORI_MIFID = {
    2 : 'Nessuno',
    7 : 'Basso',
    11 : 'Medio',
    15 : 'Medio Alto',
    22 : 'Alto',
}

def genera_codice_di_conferma_cliente():
    millitime = "%s%s" % (
        datetime.datetime.now().second,
        datetime.datetime.now().microsecond
    )
    seed = "%s%s" % (millitime, millitime)
    random.seed(seed)
    output = ''
    for i in range(0, 4):
        for j in range(0, 4):
            output = output + str(random.randrange(0, 9))
        output = output + "-"
    return output[:-1]

class AcomeaRegistrationManager(models.Manager):
    def activate_user(self, activation_key):
        # Make sure the key we're trying conforms to the pattern of a SHA1 hash;
        # if it doesn't, no point trying to look it up in the database.
        log.debug("activate_user")
        if SHA1_RE.search(activation_key):
            try:
                log.debug("activation_key : %s" % activation_key)
                profile = self.get(activation_key=activation_key)
                log.debug("profile : %s" % str(profile))
            except self.model.DoesNotExist:
                log.debug("DoesNotExist")
                return False
            if not profile.activation_key_expired():
                user = profile.user
                user.is_active = True
                user.save()
                profile.activation_key = self.model.ACTIVATED
                profile.save()
                log.debug("activated user")
                return user
        log.debug("no SHA1")
        return False

    def create_inactive_user(self, username, email, password, site, first_name, last_name, send_email=True):
        new_user = User.objects.create_user(username, email, password)
        new_user.is_active = False
        new_user.first_name = first_name
        new_user.last_name = last_name
        new_user.save()

        registration_profile = self.create_profile(new_user)

        if send_email:
            registration_profile.send_activation_email(site)

        return new_user
    create_inactive_user = transaction.commit_on_success(create_inactive_user)

    def create_profile(self, user):
        salt = sha_constructor(str(random.random())).hexdigest()[:5]
        username = user.username
        if isinstance(username, unicode):
            username = username.encode('utf-8')
        activation_key = sha_constructor(salt+username).hexdigest()
        return self.create(user=user,
            activation_key=activation_key)

    def delete_expired_users(self):
        for profile in self.all():
            if profile.activation_key_expired():
                user = profile.user
                if not user.is_active:
                    user.delete()


class AcomeaRegistrationProfile(models.Model):
    ACTIVATED = u"ALREADY_ACTIVATED"

    user = models.ForeignKey(User, unique=True, verbose_name=_('user'))
    activation_key = models.CharField(_('activation key'), max_length=40)

    objects = AcomeaRegistrationManager()

    class Meta:
        verbose_name = _('registration profile')
        verbose_name_plural = _('registration profiles')

    def __unicode__(self):
        return u"Registration information for %s" % self.user

    def activation_key_expired(self):
        expiration_date = datetime.timedelta(days=settings.ACCOUNT_ACTIVATION_DAYS)
        return self.activation_key == self.ACTIVATED or\
               (self.user.date_joined + expiration_date <= datetime.datetime.now())
    activation_key_expired.boolean = True

    def send_activation_email(self, site):
        ctx_dict = {'user' : self.user,
                    'activation_key': self.activation_key,
                    'expiration_days': settings.ACCOUNT_ACTIVATION_DAYS,
                    'site': site}
        subject = render_to_string(
            'utenti/cms_plugins/registration_activation_email_subject.txt',
            ctx_dict
        )
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())

        text_message = render_to_string('utenti/cms_plugins/registration_activation_email.txt', ctx_dict)
        html_message = render_to_string('utenti/cms_plugins/registration_activation_email.html', ctx_dict)
        try:
            msg = EmailMultiAlternatives(subject, text_message, settings.REGISTER_EMAIL_FROM, [self.user.email])
            msg.attach_alternative(html_message, "text/html")
            msg.send()
        except Exception, e:
            log.debug(u"Exception %s" % str(e))

class AcomeaProfiles(SCProfiles):
    creato_da_backoffice = models.BooleanField(default=False)
    ultima_modifica = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return "%s Profiles" % str(self.user)

    def is_lead(self):
        if self.get_profile(LeadProfile):
            return True
        return False

    def get_lead(self):
        if self.is_lead():
            return self.get_profile(LeadProfile)
        return None

    def is_cliente(self):
        if self.get_profile(ClienteProfile):
            return True
        return False

    def get_cliente(self):
        if self.is_cliente():
            return self.get_profile(ClienteProfile)
        return None

    def is_promotore(self):
        if self.get_profile(PromotoreProfile):
            return True
        return False

    def get_promotore(self):
        if self.is_promotore():
            return self.get_profile(PromotoreProfile)
        return None

    def is_utenterete(self):
        if self.get_profile(UtenteReteProfile):
            return True
        return False

    def get_utenterete(self):
        if self.is_utenterete():
            return self.get_profile(UtenteReteProfile)
        return None

    def is_acomea(self):
        if self.get_profile(AcomeaProfile):
            return True
        return False

    def get_acomea(self):
        if self.is_acomea():
            return self.get_profile(AcomeaProfile)
        return None

    def get_rac(self):
        if self.is_lead():
            return RichiestaAttivazioneCliente.objects.get(leadprofile=self.get_lead())
        return None


class LeadProfile(SCProfile):
    nickname = models.CharField(max_length=64, blank=True, null=True, unique=True)
    abilitato = models.BooleanField(default=False)
    data_di_registrazione = models.DateField(auto_now_add=True)
    data_di_nascita = models.DateField(blank=True, null=True)
    telefono = models.CharField(max_length=32, blank=True, null=True)
    indirizzo = models.CharField(max_length=128, blank=True, null=True)
    citta = models.CharField(max_length=64, blank=True, null=True)
    cap = models.CharField(max_length=5, blank=True, null=True)
    provincia = models.CharField(max_length=2, blank=True, null=True)
    hobby_uno = models.CharField(max_length=512, blank=True, null=True)
    hobby_due = models.CharField(max_length=512, blank=True, null=True)
    hobby_tre = models.CharField(max_length=512, blank=True, null=True)
    mercato_di_interesse = models.CharField(max_length=64, blank=True, null=True)
    iscrizione_alla_newsletter = models.BooleanField()
    professionale = models.BooleanField()

    def __unicode__(self):
        return "Lead: %s" % self.nominativo

    def identifier(self):
        return "lead"

    @property
    def codice_fiscale(self):
        return RichiestaAttivazioneCliente.objects.get(leadprofile=self).codice_fiscale

    @property
    def nominativo(self):
        u = self.get_user()
        return "%s %s" % (u.last_name, u.first_name)

    @property
    def datanasc(self):
        return self.data_di_nascita

    @property
    def res_indirizzo(self):
        return self.indirizzo

    @property
    def res_citta(self):
        return self.citta

    @property
    def res_prov(self):
        return self.provincia

    @property
    def res_cap(self):
        return self.cap

    @property
    def email(self):
        return self.get_user().email

    def get_rac(self):
        obj, created = RichiestaAttivazioneCliente.objects.get_or_create(
            leadprofile=self)
        return obj

class WarningRichiestaIncoerente(UserWarning):
    pass

class ExceptionSoapConnection(RuntimeError):
    pass

class RichiestaAttivazioneCliente(models.Model):
    leadprofile = models.ForeignKey(LeadProfile, unique=True)
    stato = models.CharField(max_length=128, choices=STATI_RICHIESTA_CLIENTE, default='1')
    numero_cf_errato = models.PositiveIntegerField(default=0)
    numero_non_approvazioni = models.PositiveIntegerField(default=0)
    codice_fiscale = models.CharField(max_length=16, blank=True, null=True)
    indirizzo_spedizione_alternativo = models.CharField(max_length=128, blank=True, null=True)
    citta_spedizione_alternativa = models.CharField(max_length=64, blank=True, null=True)
    cap_spedizione_alternativo = models.CharField(max_length=5, blank=True, null=True)
    codice_conferma_consultivo = models.CharField(max_length=32, blank=True, null=True)
    codice_conferma_dispositivo = models.CharField(max_length=32, blank=True, null=True)

    class Meta:
        ordering = ('stato', 'leadprofile')

    def __unicode__(self):
        return u"%s - %s" % (self.leadprofile, self.stato)

    @property
    def verbose_stato(self):
        return STATI_RAC[self.stato]

    def check_codice_fiscale(self, codice_fiscale):
        """
        controlla che l'utente lead sia un cliente di AcomeA tramite il codice fiscale
        """
        if self.stato in ('1', '3', '5'):
            ar = AcomeaClientRiservataCms()
            result = ar.get_cliente(codice_fiscale)
            if result:
                log.debug("metto la rac in 15: codice fiscale verificato")
                self.codice_fiscale = codice_fiscale
                self.stato = '15'
                #self.save()
                return True
            else:
                log.debug("lascio la rac in 1: nessuna richiesta")
                self.codice_fiscale = codice_fiscale
                self.numero_cf_errato = self.numero_cf_errato + 1
                # self.save()
                return False
        else:
            raise WarningRichiestaIncoerente()

    def richiedi_posizione_consultiva(self, codice_fiscale):
        """
        Chiedo di diventare cliente consultivo
        """
        if self.stato == '15':
            log.debug("metto la rac in 2: richiesta cliente consultivo")
            self.stato = '2'
            self.save()
            return True
        else:
            raise WarningRichiestaIncoerente()

    def accetta_richiesta_consultiva(self):
        """
        Il Backoffice accetta la richiesta
        """
        if self.stato == "2":
            log.debug("metto la rac in 14: richiesta consultiva accettata")
            self.stato = '14'
            self.codice_conferma_consultivo = genera_codice_di_conferma_cliente()
            self.codice_conferma_dispositivo = self.codice_conferma_consultivo
            self.save()
            return True
        else:
            raise WarningRichiestaIncoerente()

    def rifiuta_richiesta_consultiva(self):
        """
        Il Backoffice rifiuta la richiesta
        """
        if self.stato == "2":
            log.debug("metto la rac in 3: richiesta consultiva rifiutata")
            self.stato = '3'
            self.save()
            return True
        else:
            raise WarningRichiestaIncoerente()

    def sblocca_richiesta_consultiva(self, codice_sblocco):
        """
        l'utente sblocca la richiesta
        """
        if self.stato == "14":
            if self.codice_conferma_consultivo == codice_sblocco:
                log.debug("metto la rac in 6: richiesta consultiva sbloccata")
                self.stato = '6'
                self.save()
                lead_profile = self.leadprofile.get_profiles()
                try:
                    lead_profile.get_profile(ClienteProfile)
                except:
                    lead_profile.add_profile(
                        ClienteProfile,
                        fields_dict={'dispositivo' : False}
                    )
                return True
            else:
                log.debug("lascio la rac in 14: richiesta consultiva accettata")
                return False
        else:
            raise WarningRichiestaIncoerente()

    def trasforma_consultiva_in_dispositiva(self, codice_sblocco):
        """
        L'utente trasforma posizione da consultiva a dispositiva
        """
        if self.stato == '6':
            if self.codice_conferma_consultivo == codice_sblocco:
                log.debug("metto la rac in 9: richiesta dispositiva sbloccata")
                self.stato = '9'
                self.save()
                lead_profile = self.leadprofile.get_profiles()
                cliente_profile = lead_profile.get_profile(ClienteProfile)
                cliente_profile.dispositivo = True
                cliente_profile.save()
                return True
            else:
                log.debug("lascio la rac in 6: richiesta consultiva sbloccata")
                return False


    def richiedi_posizione_dispositiva(self, codice_fiscale):
        """
        Chiedo di diventare cliente consultivo
        """
        if self.stato == '15':
            log.debug("metto la rac in 4: richiesta cliente dispositiva")
            self.stato = '4'
            self.save()
            return True
        else:
            raise WarningRichiestaIncoerente()

    def accetta_richiesta_dispositiva(self):
        """
        Il Backoffice accetta la richiesta
        """
        if self.stato == "4":
            log.debug("metto la rac in 8: richiesta dispositiva accettata")
            self.codice_conferma_dispositivo = genera_codice_di_conferma_cliente()
            self.codice_conferma_consultivo = self.codice_conferma_dispositivo
            self.stato = '8'
            self.save()
            return True
        else:
            raise WarningRichiestaIncoerente()

    def rifiuta_richiesta_dispositiva(self):
        """
        Il Backoffice rifiuta la richiesta
        """
        if self.stato == "4":
            log.debug("metto la rac in 5: richiesta dispositiva rifiutata")
            self.stato = '5'
            self.save()
            return True
        else:
            raise WarningRichiestaIncoerente()

    def sblocca_richiesta_dispositiva(self, codice_sblocco):
        """
        l'utente sblocca la richiesta
        """
        if self.stato == "8":
            if self.codice_conferma_consultivo == codice_sblocco:
                log.debug("metto la rac in 9: richiesta dispositiva sbloccata")
                self.stato = '9'
                self.save()
                lead_profile = self.leadprofile.get_profiles()
                try:
                    lead_profile.get_profile(ClienteProfile)
                except:
                    lead_profile.add_profile(
                        ClienteProfile,
                        fields_dict={'dispositivo' : True}
                    )
                return True
            else:
                log.debug("lascio la rac in 8: richiesta dispositiva accettata")
                return False
        else:
            raise WarningRichiestaIncoerente()

    def trasforma_dispositiva_in_consultiva(self):
        """
        L'utente trasforma posizione da dispositiva a consultiva
        """
        if self.stato == '9':
            log.debug("metto la rac in 9: richiesta dispositiva sbloccata")
            self.stato = '6'
            self.save()
            lead_profile = self.leadprofile.get_profiles()
            cliente_profile = lead_profile.get_profile(ClienteProfile)
            cliente_profile.dispositivo = False
            cliente_profile.save()
            return True
        else:
            raise WarningRichiestaIncoerente()


    def sblocca_richiesta_cliente(self):
        if self.leadprofile.abilitato == False and self.numero_cf_errato > 2:
            self.numero_cf_errato = 0
            self.leadprofile.abilitato = True
            self.save()

class ClienteProfile(SCProfile):
    abilitato = models.BooleanField(default=False)
    dispositivo = models.BooleanField(default=False)
    mifid_compilato = models.BooleanField(default=False)

    def __unicode__(self):
        return "Cliente: %s" % self.get_codice_albero()

    def identifier(self):
        return "cliente"

    def get_codice_albero(self):
        try:
            return self.get_profiles().get_profile(LeadProfile).codice_fiscale
        except Exception:
            pass

    def get_date_mifid(self):
#        output = []
#        for a in Answer.objects.filter(user__exact=self.get_user()):
#            d = datetime.date(a.submission_date.year, a.submission_date.month,
#                a.submission_date.day)
#            if output.count(d) == 0:
#                output.append(d)
#        output.sort()
#        return output
        raise NotImplementedError

    def get_punteggio_mifid(self, in_data=None):
        # nella nuova versione il mifid viene gestito esternamente
#        if not self.mifid_compilato:
#            return None
#        score = 0
#        for a in Answer.objects.filter(user__exact=self.get_user()):
#            score += a.score
#        lista = VALORI_MIFID.keys()
#        lista.sort()
#        output = VALORI_MIFID[lista[0]]
#        for k in lista:
#            if score >= k:
#                output = VALORI_MIFID[k]
#        return output
        #raise NotImplementedError
        return None

    def get_risposte_mifid(self, in_data=None, question=None):
#        if not self.mifid_compilato:
#            return None
#        if not in_data:
#            return Answer.objects.filter(user__exact=self.get_user())
#        else:
#            output =  Answer.objects.filter(user__exact=self.get_user(),
#                submission_date__startswith=in_data)
#            if question:
#                output = output.filter(question__exact=question)
#            return output
        raise NotImplementedError

    @staticmethod
    def get_user_from_codice_albero(codice):
        rac = RichiestaAttivazioneCliente.objects.get(codice_fiscale=codice)
        return rac.leadprofile.get_user()

    @property
    def is_consultivo_bloccato(self):
        rac = self.get_profiles().get_profile(LeadProfile).get_rac()
        # se non sono segnato come dispositivo ma ho il codice sblocco dispositivo sono un dispositivo
        # che ha chiesto il blocco ( e non ho bisogno di reinviare il contratto per sbloccarmi )
        if rac.codice_conferma_dispositivo and self.dispositivo == False:
            return True
        return False


class PromotoreProfile(SCProfile):
    abilitato = models.BooleanField(default=True)
    codice = models.CharField(max_length=32, unique=True)

    def __unicode__(self):
        return "Promotore: %s" % self.get_codice_albero()

    def identifier(self):
        return "promotore"

    def get_codice_albero(self):
        return self.codice

    @staticmethod
    def get_user_from_codice_albero(codice):
        return PromotoreProfile.objects.get(codice=codice).get_user()

class Rete(models.Model):
    nome = models.CharField(max_length=64)
    codice = models.CharField(max_length=32, unique=True)

    class Meta:
        verbose_name_plural = 'Reti'

    def __unicode__(self):
        return self.nome

class UtenteReteProfile(SCProfile):
    abilitato = models.BooleanField(default=True)
    rete = models.ForeignKey(Rete)

    def __unicode__(self):
        return "UtenteRete: %s" % self.get_codice_albero()

    def get_codice_albero(self):
        return self.rete.codice

    @staticmethod
    def get_user_from_codice_albero(codice):
        rete = Rete.objects.get(codice=codice)
        return UtenteReteProfile.objects.get(rete=rete).get_user()

class AcomeaProfile(SCProfile):
    abilitato = models.BooleanField(default=True)

    def get_codice_albero(self):
        return "CLIDIRETTI"

class OneTimePassword(models.Model):
    timestamp = models.DateTimeField(auto_now_add=True)
    otp = models.CharField(max_length=64, unique=True)
    sessionid = models.CharField(max_length=64)
    utente_richiedente = models.ForeignKey(User)
    codice_richiedente = models.CharField(max_length=32)
    usata = models.BooleanField(default=False)

def add_lead_profile_to_user(user, data):
    """
    Aggiunge un profilo di tipo Lead al normale utente django e crea un oggetto di richiesta attivazione
    Cliente allo stato 1 (nessuna richiesta)
    """
    log.debug("add_lead_profile_to_user")
    log.debug("user: %s" % user)
    log.debug("data: %s" % data)
    try:
        profiles = AcomeaProfiles.objects.get(user=user)
        if data.has_key('email'):
            data.pop('email')
        profiles.add_profile("LeadProfile", "acomea.utenti.models", data)
        lead = profiles.get_profile("LeadProfile", "acomea.utenti.models")
        log.debug("add_lead_to_profiles: aggiunto LeadProfile a %s" % user)
        rac, created = RichiestaAttivazioneCliente.objects.get_or_create(leadprofile=lead)
        log.debug("add_lead_to_profiles: aggiunta RichiestaAttivazioneCliente con id %s" % rac.id)
    except Exception, e:
        log.debug("add_lead_to_profiles: Exception %s" % str(e))

def add_cliente_profile_to_user(user, data):
    """
    Aggiunge un profilo di tipo Cliente al normale utente django
    """
    log.debug("add_cliente_profile_to_user")
    log.debug("user: %s" % user)
    log.debug("data: %s" % data)
    try:
        profiles = AcomeaProfiles.objects.get(user=user)
        profiles.add_profile("ClienteProfile", "acomea.utenti.models", data)
        log.debug("add_cliente_profile_to_user: aggiunto ClienteProfile a %s" % user)
    except Exception, e:
        log.debug("add_lead_to_profiles: Exception %s" % str(e))

def create_user_profiles(sender, user, request=None, **kwargs):
    log.debug("create_user_profiles")
    log.debug("user: %s" % user)
    try:
        p, created = AcomeaProfiles.objects.get_or_create(user=user)
        if created:
            log.debug("create_user_profiles: creato AcomeaProfiles per %s" % user)
    except Exception, e:
        log.debug("create_user_profiles: Exception %s" % str(e))

def create_user_profiles_from_ju(instance, raw, created, **kwargs):
    log.debug("create_user_profiles_from_ju")
    log.debug("user: %s" % instance)
    if created:
        try:
            p, c = AcomeaProfiles.objects.get_or_create(user=instance)
            if c:
                log.debug("create_user_profiles: creato AcomeaProfiles per %s" % instance)
        except Exception, e:
            log.debug("create_user_profiles: Exception %s" % str(e))

def enable_cliente_after_mifid(survey, request, **kwargs):
    log.debug("enable_cliente_after_mifid")
    log.debug("user: %s" % request.user)
    try:
        c = request.user.get_profile().get_profile(ClienteProfile)
        if c.abilitato == False and c.mifid_compilato == False:
            c.abilitato = True
            c.mifid_compilato = True
            c.save()
    except Exception, e:
        log.debug("enable_cliente_after_mifid: Exception %s" % str(e))

def dump_utente_xml_backup(sender, instance, **kwargs):
    log.debug("dump_utente_xml_backup: %s" % str(instance))
    try:
        p = AcomeaProfiles.objects.get(user=instance)
        f = open("%suploads/backup_dumps/%s-%s.xml" % (getattr(settings, 'DOCUMENT_ROOT', '/tmp/'),
                                                       instance.email,
                                                       datetime.date.today()), 'w')
        f.write(render_to_string('utenti/backoffice/backup_utente.xml', locals()))
        f.close()
    except AcomeaProfiles.DoesNotExist:
        pass

def dump_profilo_xml_backup(sender, instance, **kwargs):
    log.debug("dump_profilo_xml_backup: %s" % str(instance))
    try:
        f = open("%suploads/backup_dumps/%s-%s-%s.xml" % (getattr(settings, 'DOCUMENT_ROOT', '/tmp/'),
                                                          instance.__class__.__name__,
                                                          instance.get_user().email,
                                                          datetime.date.today()), 'w')
        p = AcomeaProfiles.objects.get(user=instance.get_user())
        f.write(render_to_string('utenti/backoffice/backup_utente.xml', locals()))
        f.close()
    except AcomeaProfiles.DoesNotExist:
        pass


#user_registered.connect(create_user_profiles)
#post_save.connect(create_user_profiles_from_ju, sender=User)
#pre_delete.connect(dump_utente_xml_backup, sender=User)
#pre_delete.connect(dump_profilo_xml_backup, sender=LeadProfile)
#pre_delete.connect(dump_profilo_xml_backup, sender=ClienteProfile)
#pre_delete.connect(dump_profilo_xml_backup, sender=PromotoreProfile)

#import cms stuffs
if settings.CMS_ENABLED:
    import acomea.utenti.cms.plugins
    #import acomea.utenti.cms.app


# monkey patch for replace original delete with obfuscate (#REPLACE_DELETE_OBFUSCATE)
def obfuscate(self):

    email = self.email

    # uso u'..' per controllo su ldap
    self.first_name = u'deleted'
    self.last_name = u'deleted'
    # username non cambia per compatibilità con Gimme5
    # self.username = u'deleted_{}'.format(self.pk)
    self.email = u'deleted_{}@deleted.com'.format(self.pk)
    self.is_active = False
    self.save()

    if hasattr(self, 'contact'):
        self.contact.obfuscate()

    if hasattr(self, 'profilo'):
        self.profilo.obfuscate()
        if hasattr(self.profilo, 'rac'):
            self.profilo.rac.obfuscate()

    mailup_client = MailUpClientSingleton(
        client_id=settings.ACOMEA_MAILUP_CLIENT_ID,
        client_secret=settings.ACOMEA_MAILUP_SECRET,
        username=settings.ACOMEA_MAILUP_USERNAME,
        password=settings.ACOMEA_MAILUP_PASSWORD,
    )

    provider = MailUpComponentProvider(client=mailup_client)
    try:
        recipient = provider.get_recipient(settings.ACOMEA_MAILUP_LIST_ID, email=email)
        recipient.set_fields({
            'Nome': self.first_name,
            'Cognome': self.last_name,
            'ID_USER': self.username
        })
        recipient.email = self.email
        recipient.save()

        recipient.unsubscribe_to_list(settings.ACOMEA_MAILUP_LIST_ID)
    except RecipientNotFoundException:
        log.warning("OBFUSCATE The recipient {email} not found in MAIL_UP".format(email=email))


User.original_delete = User.delete
User.delete = User.obfuscate = obfuscate
